  MODULE fun_mod

!-----------------------------------------------------------------------
! Module containing procedures for the FUN (Fixation and Uptake of Nitrogen)
! model, which deals with plant uptake of N.
!-----------------------------------------------------------------------

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: fun_driver

  CONTAINS

!###############################################################################
!###############################################################################

  SUBROUTINE fun_driver( a_step,funCount,land_pts,npft,ntiles                &
                 ,fun_period,timestep                                        &
                 ,tile_index,tile_pts,ammnSI,critSI,esoil_tile,factor_evap   &
                 ,frac,g_leaf_phen,canht_ft,npp_ft,rootFrac,smvcst,soilNSI         &
                 ,sthf,sthu,t_soil,wt_ext_tile                               &
                 ,npp_ft_acc,npp_acc_fun,resp_p,resp_p_ft,transp_acc_fun     &
                 ,fun_call,gb_nh4_uptake,gb_no3_uptake                                &
                 ,leaf_litter_cn_ratio                                       &
                 ,Nactive,Ndemand,Nfix,Npassive,Nretrans,Nuptake             &
                 ,npp_Nactive,npp_Nfix,npp_Nretrans,npp_Nuptake )

!-----------------------------------------------------------------------
! Top level routine for the FUN sub-model.
!-----------------------------------------------------------------------

  USE ancil_info, ONLY :  &
!  imported scalars with intent(in)
     nzSoilC,sm_levels

  IMPLICIT NONE

!-------------------------------------------------------------------------------
! Scalar arguments with intent(in)
!-------------------------------------------------------------------------------
  INTEGER, INTENT(in) :: a_step        !  atmospheric timestep number.     
  INTEGER, INTENT(in) :: funCount      !  counter of calls to FUN
  INTEGER, INTENT(in) :: land_pts      !  number of land_pts
  INTEGER, INTENT(in) :: npft          !  number of plant functional types
  INTEGER, INTENT(in) :: ntiles        !  number of tiles
  INTEGER, INTENT(in) :: fun_period    !  FUN period (days).
  REAL, INTENT(in) :: timestep         !  atmospheric timestep length (s)

!-------------------------------------------------------------------------------
! Array arguments with intent(in)
!-------------------------------------------------------------------------------
  INTEGER, INTENT(in) :: tile_index(land_pts,ntiles)
!                                Indices of land points for each tile
  INTEGER, INTENT(in) :: tile_pts(ntiles)
!                                Number of land points for each tile
  REAL, INTENT(in) :: ammnSI(land_pts,nzSoilC)  !  amount of NH4 in the soil (kgN m-2)
  REAL, INTENT(in) :: critSI(land_pts,nzSoilC)  !  critical (=minimum) N content of soil
!                                           layers (kgN m-2)
  REAL, INTENT(in) :: esoil_tile(land_pts,npft)
!                                Soil surface evaporation and transpiration
!                                (kg m-2 s-1)
  REAL, INTENT(in) :: factor_evap(land_pts,npft)! Ratio of transpiration 
!                                                 to total soil evaporation
  REAL, INTENT(in) :: frac(land_pts,npft)       !  fractional cover
  REAL, INTENT(in) :: g_leaf_phen(land_pts,npft)!  leaf turnover rate (kg leaves off/
!                                                  kg leaves total/ (360days) 
  REAL, INTENT(in) :: canht_ft(land_pts,npft)   !  height of canopy (m)
  REAL, INTENT(in) :: npp_ft(land_pts,npft)     !  Net primary productivity
!                               assuming no expenditure on N uptake (kgC m-2 s-1)
  REAL, INTENT(in) :: rootFrac(npft,sm_levels)  !  fraction of roots in each layer
  REAL, INTENT(in) :: smvcst(land_pts,sm_levels)!  volumetric soil moisture at
!                                                  saturation 
  REAL, INTENT(in) :: soilNSI(land_pts,nzSoilC) !  amount of nitrate in the soil
!                                                    (kgN m-2)
  REAL, INTENT(in) :: sthf(land_pts,sm_levels)  !  frozen soil moisture content
!                                                    as a fraction of saturation
  REAL, INTENT(in) :: sthu(land_pts,sm_levels)  !  unfrozen soil moisture content
!                                                    as a fraction of saturation
  REAL, INTENT(in) :: t_soil(land_pts,sm_levels)!  soil temperature (K)
  REAL, INTENT(in) :: wt_ext_tile(land_pts,sm_levels,npft) !  fraction of
!                                transpiration extracted from each layer
 
!-------------------------------------------------------------------------------
! Array arguments with intent(inout)
!-------------------------------------------------------------------------------
  REAL, INTENT(inout) :: NPP_FT_ACC(LAND_PTS,NPFT)   ! Accumulated NPP since
!                                           previous call to TRIFFID (kg C/m2)
  REAL, INTENT(inout) :: npp_acc_fun(land_pts,npft)  ! NPP accumulated over 
!                                                        FUN timestep (kgC m-2)
  REAL, INTENT(inout) :: resp_p(land_pts)            !  Gridbox mean plant 
!                                                       respiration (kgC m-2 s-1)
  REAL, INTENT(inout) :: resp_p_ft(land_pts,npft)    !  Plant respiration
!                                                          (kgC m-2 s-1)

  REAL, INTENT(inout) :: transp_acc_fun(land_pts,npft) ! Transpiration
!                              accumulated over FUN timestep (kg m-2)

!-------------------------------------------------------------------------------
! Scalar arguments with intent(out)
!-------------------------------------------------------------------------------
  LOGICAL, INTENT(out) :: fun_call  !  TRUE on timesteps when FUN is called

!-------------------------------------------------------------------------------
! Array arguments with intent(out)
! The majority of these are for diagnostic purposes only.
!-------------------------------------------------------------------------------
  REAL, INTENT(out) :: gb_nh4_uptake(land_pts,nzsoilC)  !  gridbox NH4 uptake
!                                                            (kgN m-2)
  REAL, INTENT(out) :: gb_no3_uptake(land_pts,nzsoilC)  !  gridbox NO3 uptake
!                                                            (kgN m-2)
  REAL, INTENT(out) :: leaf_litter_cn_ratio(land_pts,npft)
!                           C:N ratio of leaf litter (kgN/kgC)
  REAL, INTENT(out) :: Nactive(land_pts,npft)
!                           N acquired by active uptake (kgN m-2 s-1)
  REAL, INTENT(out) :: Ndemand(land_pts,npft)
!                           N demand from plants (kgN m-2 s-1)
  REAL, INTENT(out) :: Nfix(land_pts,npft)
!                           N acquired by fixing (kgN m-2 s-1)
  REAL, INTENT(out) :: Npassive(land_pts,npft)
!                           N acquired by passive uptake (kgN m-2 s-1)
  REAL, INTENT(out) :: npp_Nactive(land_pts,npft)
!                           NPP reduction to pay for active uptake of N
!                           (kgC m-2 s-1)
  REAL, INTENT(out) :: npp_Nfix(land_pts,npft)
!                           NPP reduction to pay for fixing N  (kgC m-2 s-1)
  REAL, INTENT(out) :: npp_Nretrans(land_pts,npft)
!                           NPP reduction to pay for retranslocation of N
!                           (kgC m-2 s-1)
  REAL, INTENT(out) :: npp_Nuptake(land_pts,npft)
!                           Total NPP reduction to pay for N (kgC m-2 s-1)
  REAL, INTENT(out) :: Nretrans(land_pts,npft)
!                           N acquired by retranslocation (kgN m-2 s-1)
  REAL, INTENT(out) :: Nuptake(land_pts,npft)
!                           Total N acquired by plants through passive and
!                           active uptake, fixing and retranslocation
!                           (kgN m-2 s-1)
    
!-----------------------------------------------------------------------
! Local scalar variables.    
!-----------------------------------------------------------------------
  INTEGER :: nstep_fun    ! Number of atmospheric timesteps between
!                           calls to FUN
  INTEGER :: l,n          ! loop counters

!-----------------------------------------------------------------------
! Increment accumulations for FUN.
!-----------------------------------------------------------------------
  CALL fun_accum( land_pts,npft,ntiles,timestep  &
                 ,tile_index,tile_pts,esoil_tile,factor_evap,npp_ft  &
                 ,npp_acc_fun,transp_acc_fun )

!-----------------------------------------------------------------------
! Calculate the number of atmospheric timesteps between calls to FUN.
!-----------------------------------------------------------------------
  nstep_fun = INT( 86400.0*fun_period / timestep )
  fun_call = .FALSE.

!-----------------------------------------------------------------------
! Call FUN routine to calculate plant uptake and NPP expenditure.
!-----------------------------------------------------------------------
  IF ( MOD(a_step,nstep_fun) == 0 ) THEN

!-----------------------------------------------------------------------
!   Set flag to show that FUN is called this timestep.
!-----------------------------------------------------------------------
    fun_call = .TRUE.

!-----------------------------------------------------------------------
!   Calculate plant uptake of N.
!-----------------------------------------------------------------------
    CALL FUN_UPTAKE_N( funCount,land_pts,tile_index(:,1:npft)          &
                      ,tile_pts(1:npft),frac(:,1:npft),g_leaf_phen     &
                      ,canht_ft,rootFrac,wt_ext_tile,sthf,sthu,smvcst        &
                      ,t_soil,ammnSI,critSI,soilNSI,transp_acc_fun     &
                      ,npp_acc_fun,resp_p,resp_p_ft                    &
                      ,gb_nh4_uptake,gb_no3_uptake                     &
                      ,leaf_litter_cn_ratio                            &
                      ,Nactive,Ndemand,Nfix,Npassive,Nretrans,Nuptake  &
                      ,npp_Nactive,npp_Nfix,npp_Nretrans,npp_Nuptake )

!-----------------------------------------------------------------------
!   Increment TRIFFID accumulation of NPP.
!-----------------------------------------------------------------------
    DO N=1,NPFT
      DO L=1,LAND_PTS
        NPP_FT_ACC(L,N) = NPP_FT_ACC(L,N) + NPP_ACC_FUN(L,N)
      ENDDO
    ENDDO

!-----------------------------------------------------------------------
!   Reset FUN accumulations.
!-----------------------------------------------------------------------
    npp_acc_fun(:,:) = 0.0
    transp_acc_fun(:,:) = 0.0

  ENDIF  !  call fun

  END SUBROUTINE fun_driver
!###############################################################################
!###############################################################################

  SUBROUTINE fun_accum( land_pts,npft,ntiles,timestep  &
                       ,tile_index,tile_pts,esoil_tile,factor_evap,npp_ft  &
                       ,npp_acc_fun,transp_acc_fun )

!-----------------------------------------------------------------------
! Add to accumulations used by the FUN model.
!-----------------------------------------------------------------------

!-------------------------------------------------------------------------------
! Scalar arguments with intent(in)
!-------------------------------------------------------------------------------
  INTEGER, INTENT(in) :: land_pts    !  number of land_pts
  INTEGER, INTENT(in) :: npft        !  number of plant functional types
  INTEGER, INTENT(in) :: ntiles      !  number of tiles
  REAL, INTENT(in) :: timestep       !  timestep length (s)

!-------------------------------------------------------------------------------
! Array arguments with intent(in)
!-------------------------------------------------------------------------------
  INTEGER, INTENT(in) :: tile_index(land_pts,ntiles)
!                                Indices of land points for each tile
  INTEGER, INTENT(in) :: tile_pts(ntiles)
!                                Number of land points for each tile
  REAL, INTENT(in) :: esoil_tile(land_pts,npft)
!                                Soil surface evaporation and transpiration
!                                (kg m-2 s-1)
  REAL, INTENT(in) :: factor_evap(land_pts,npft)
!                                Ratio of transpiration to total soil evaporation
  REAL, INTENT(in) :: npp_ft(land_pts,npft)
!                                Net primary productivity (kgC m-2 s-1)
 
!-------------------------------------------------------------------------------
! Array arguments with intent(inout)
!-------------------------------------------------------------------------------
  REAL, INTENT(inout) :: npp_acc_fun(land_pts,npft)
!                                Accumulated NPP over FUN timestep (kgC m-2)
  REAL, INTENT(inout) :: transp_acc_fun(land_pts,npft)
!                                Accumulated transpiration over FUN timestep (kg m-2)

!-------------------------------------------------------------------------------
! Local scalar variables
!-------------------------------------------------------------------------------
  INTEGER :: j  !  loop counter
  INTEGER :: l  !  index
  INTEGER :: n  !  loop counter
!-------------------------------------------------------------------------------
! Increment the accumulated NPP and transpiration.
!-------------------------------------------------------------------------------
  DO n=1,npft
    DO j=1,tile_pts(n)
      l = tile_index(j,n)
      npp_acc_fun(l,n) = npp_acc_fun(l,n) + npp_ft(l,n) * timestep
      transp_acc_fun(l,n) = transp_acc_fun(l,n) +   &
                    esoil_tile(l,n) * factor_evap(l,n) * timestep
!      transp_acc_fun(l,n) = 100.0  !  100 mm d-1  cxyz
    ENDDO
  ENDDO

  END SUBROUTINE fun_accum

!###############################################################################
!###############################################################################
!###############################################################################

  SUBROUTINE fun_uptake_n( funCount,land_pts,tile_index  &
                         ,tile_pts,frac,g_leaf_phen  &
                         ,canht_ft,rootFrac,wt_ext_tile,sthf,sthu,smvcst  &
                         ,t_soil,ammnSI,critSI,soilNSI,transp_acc_fun  &
                         ,npp_acc_fun,resp_p,resp_p_ft  &
                         ,gb_nh4_uptake,gb_no3_uptake  &
                         ,leaf_litter_cn_ratio                            &
                         ,Nactive,Ndemand,Nfix,Npassive,Nretrans,Nuptake  &
                         ,npp_Nactive,npp_Nfix,npp_Nretrans,npp_Nuptake )
  
!----------------------------------------------------------------------------------
! Nitrogen extraction (passive, active, N fixation) by vegetation
! Designed for use with TRIFFID DVGM and ECOSSE soil model in JULES.
! Code designed by Rosie Fisher, Josh Fisher, Stephen Sitch, Chris Huntingford and
! Douglas Clark.
!----------------------------------------------------------------------------------
  
! Philosophy of N uptake model.
! This model calculates first the passive uptake of N via the transpiration stream
! then, if that uptake is insufficient to satisfy the demand from NPP, the remaining
! NPP is used to pay for N uptake by one or more of fixation, active uptake and
! retranslocation. The N extracted by this method must be the same as the N used
! for growth (n_grow=n_uptake) hence we calculate the optimum amount of carbon
! supplied to the roots/nodules to satisfy this constraint.
!
! NB There is no representation of N15 in this subroutine, despite the fact
! that ECOSSE can have N15 in both NO3 and NH4.
  
!-------------------------------------------------------------------------------
  USE ancil_info, ONLY :  &
!  imported scalars with intent(in)
     nzSoilC,sm_levels

  USE C_0_DG_C,ONLY :  &
!  imported scalar parameters
     zeroDegC

  USE c_densty, ONLY :  &
!  imported scalar parameters
     rho_water

  USE ecosse_param, ONLY : &
!  imported arays with intent(in)
     dzSoilC

  USE ecosse_utils, ONLY :  &
!  imported procedures
     ecosseLayers,findLayer

  USE fun_param, ONLY :  &
!  imported scalars with intent(in)
     a_fix,b_fix,c_fix,kc_active,kn_active,kr_resorb   &
    ,nstep=>nstepFUN,s_fix,fun_period

  USE nstypes, ONLY :  &
!  imported scalars with intent(in)
     npft,ntype

  USE p_s_parms, ONLY : &
!  imported arays with intent(in)
     dzsoil

  USE pftparm, ONLY :  &
!  imported arrays with intent(in)
     a_wl,a_ws,b_wl,eta_sl,fixer,kpar,nl0,nr_nl,ns_nl,sigl

  USE switches, ONLY :  &
!  imported scalars with intent(in)
     can_rad_mod

  USE timeconst, ONLY :  &
!  imported scalar parameters with intent(in)
     isecInDay,isecInYear  !  isecInYear for debug

  IMPLICIT NONE

!-------------------------------------------------------------------------------
! Local scalar parameters.
!-------------------------------------------------------------------------------
  INTEGER, PARAMETER :: ncost = 4   !  number of processes through which N can
!            be extracted. Each of these has an iCostXX variable below. Each
!            process has a unique value of iCostXX from the set [1,ncost].
  INTEGER, PARAMETER :: icostActiveNH4 = 1 !  process number for active uptake of NH4
  INTEGER, PARAMETER :: icostActiveNO3 = 2 !  process number for active uptake of NO3
  INTEGER, PARAMETER :: icostFix = 3       !  process number for fixing
  INTEGER, PARAMETER :: icostRetrans = 4   !  process number for retranslocation

  REAL, PARAMETER :: big_cost = 1000000.   !  an arbitrary large cost (kgC/kgN)
  REAL, PARAMETER :: litter_cn_max = 3000. !  maximum allowed value of C:N in
!                                               leaf litter

!-------------------------------------------------------------------------------
! Scalar arguments with intent(in)
!-------------------------------------------------------------------------------
  INTEGER, INTENT(in) :: funCount   !  counter of calls to FUN
  INTEGER, INTENT(in) :: land_pts   !  number of land points

!-------------------------------------------------------------------------------
! Array arguments with intent(in)
!-------------------------------------------------------------------------------
  INTEGER, INTENT(in) :: tile_index(land_pts,npft) ! Indices of land points
!                                            that include the nth surface type.
  INTEGER, INTENT(in) :: tile_pts(npft)            !  Number of land points that
!                                    include the nth surface type.
  REAL, INTENT(in) :: frac(land_pts,npft)       !  fractional cover
  REAL, INTENT(in) :: g_leaf_phen(land_pts,npft)!  leaf turnover rate (kg leaves off/
!                                                    kg leaves total/ (360days) 
  REAL, INTENT(in) :: canht_ft(land_pts,npft)   !  height of canopy of tile in question (m)
  REAL, INTENT(in) :: rootFrac(npft,sm_levels)  !  fraction of roots in each layer
  REAL, INTENT(in) :: wt_ext_tile(land_pts,sm_levels,npft)  !  fraction of transpiration extracted
!                                                 from each layer
  REAL, INTENT(in) :: sthf(land_pts,sm_levels)  !  frozen soil moisture content
!                                                    as a fraction of saturation
  REAL, INTENT(in) :: sthu(land_pts,sm_levels)  !  unfrozen soil moisture content
!                                                    as a fraction of saturation
  REAL, INTENT(in) :: smvcst(land_pts,sm_levels)!  volumetric soil moisture at saturation
  REAL, INTENT(in) :: t_soil(land_pts,sm_levels)!  soil temperature (K)
  REAL, INTENT(in) :: ammnSI(land_pts,nzSoilC)  !  amount of NH4 in the soil (kgN m-2)
  REAL, INTENT(in) :: critSI(land_pts,nzSoilC)  !  critical (=minimum) N content of soil
!                                           layers (kgN m-2) 
  REAL, INTENT(in) :: soilNSI(land_pts,nzSoilC)   !  amount of nitrate in the soil
!                                           (kgN m-2)
  REAL, INTENT(in) :: transp_acc_fun(land_pts,npft)! Accumulated transpiration
!                                             over FUN timestep (kg m-2)

!-------------------------------------------------------------------------------
! Array arguments with intent(inout)
!-------------------------------------------------------------------------------
  REAL, INTENT(inout) :: npp_acc_fun(land_pts,npft)
!                           Accumulated NPP over FUN timestep (kgC m-2)
!       On entry, this has been calculated assuming no N limitation.
!       On exit, this has been reduced to account for expenditure on N uptake.
  REAL, INTENT(inout) :: resp_p(land_pts)
!                           Gridbox mean plant respiration (kgC m-2 s-1)
  REAL, INTENT(inout) :: resp_p_ft(land_pts,npft)
!                           Plant respiration for this tile (kgC m-2 s-1)

!-------------------------------------------------------------------------------
! Array arguments with intent(out)
! The majority of these are for diagnostic purposes only.
!-------------------------------------------------------------------------------
  REAL, INTENT(out) :: gb_nh4_uptake(land_pts,nzsoilC)  !  gridbox NH4 uptake
!                                                            (kgN m-2)
  REAL, INTENT(out) :: gb_no3_uptake(land_pts,nzsoilC)  !  gridbox NO3 uptake
!                                                            (kgN m-2)
  REAL, INTENT(out) :: leaf_litter_cn_ratio(land_pts,npft)
!                           C:N ratio of leaf litter (kgN/kgC)
  REAL, INTENT(out) :: Nactive(land_pts,npft)
!                           N acquired by active uptake (kgN m-2 s-1)
  REAL, INTENT(out) :: Ndemand(land_pts,npft)
!                           N demand from plants (kgN m-2 s-1)
  REAL, INTENT(out) :: Nfix(land_pts,npft)
!                           N acquired by fixing (kgN m-2 s-1)
  REAL, INTENT(out) :: Npassive(land_pts,npft)
!                           N acquired by passive uptake (kgN m-2 s-1)
  REAL, INTENT(out) :: npp_Nactive(land_pts,npft)
!                           NPP reduction to pay for active uptake of N
!                           (kgC m-2 s-1)
  REAL, INTENT(out) :: npp_Nfix(land_pts,npft)
!                           NPP reduction to pay for fixing N  (kgC m-2 s-1)
  REAL, INTENT(out) :: npp_Nretrans(land_pts,npft)
!                           NPP reduction to pay for retranslocation of N
!                           (kgC m-2 s-1)
  REAL, INTENT(out) :: npp_Nuptake(land_pts,npft)
!                           Total NPP reduction to pay for N (kgC m-2 s-1)
  REAL, INTENT(out) :: Nretrans(land_pts,npft)
!                           N acquired by retranslocation (kgN m-2 s-1)
  REAL, INTENT(out) :: Nuptake(land_pts,npft)
!                           Total N acquired by plants through passive and
!                           active uptake, fixing and retranslocation
!                           (kgN m-2 s-1)

!-------------------------------------------------------------------------------
! Local scalar variables.
!-------------------------------------------------------------------------------
  INTEGER :: ft,i,icost,ipft,istep,iz,j,j1,j2,jz,l  !  loop counters/work
  REAL :: c_turnover       !  C lost in the turnover of leaves over timestep
!                               i.e. C added to leaf litter (kgC m-2)
  REAL :: currentCN        !  average C:N ratio of whole plant (kgC/kgN)
  REAL :: dn               !  increment of N (kgN m-2)
  REAL :: dnpp             !  increment of NPP (kgC m-2)
  REAL :: excess           !  excess N taken up by transpiration (kgN m-2)
  REAL :: fpar             !  FPAR of tile (frac)
  REAL :: lai              !  LAI (m2 m-2)
  REAL :: lai_bal          !  balanced lai (m2 m-2)
  REAL :: leaf             !  leaf biomass (kgC m-2)
  REAL :: multi            !  multiplier to convert rate from per timestep to per s.
  REAL :: n_active_acc     !  N acquired by active uptake (kgN m-2)
  REAL :: n_fix_acc        !  N acquired by fixing (kgN m-2)
  REAL :: n_passive_acc    !  N acquired by passive uptake (kgN m-2)
  REAL :: n_retrans_acc    !  N acquired by retranslocation (kgN m-2)
  REAL :: n_demand_step    !  N required by input NPP over one step (kgN m-2)
  REAL :: n_leaf           !  total N of living leaves (kgN m-2)
  REAL :: n_litter_min     !  mimimum allowed value of leaf litter N (kgN m-2)
  REAL :: n_passive_step   !  N taken up through transpiration over substep (kgN m-2)
  REAL :: n_root           !  total N of roots  (kgN m-2)
  REAL :: n_wood           !  total N of wood (kgN m-2)
  REAL :: n_falling_leaves !  N stored in leaves that are about to fall off (kgN m-2)
  REAL :: nl               !  N concentration of leaf (kgN m-2 leaf)
  REAL :: nl_bal           !  'Balanced' N of canopy (kgN m-2)
  REAL :: npp_active_acc   !  NPP used for active uptake of N (kgC m-2)
  REAL :: npp_fix_acc      !  amount of NPP used by fixation nodules (kgC m-2)
  REAL :: npp_remaining    !  remaining (unspent) NPP over the FUN timestep
!                               that has still to be matched with N (kgC m-2)
  REAL :: npp_remaining_step !  remaining (unspent) NPP over the sub-step
!                               that has still to be matched with N (kgC m-2)
  REAL :: npp_retrans_acc  !  NPP used for the extraction of N from leaves.
!                               (kgC m-2)
  REAL :: npp_step         !  NPP per sub-step (kgC m-2)
  REAL :: root             !  root biomass (kgC m-2)
  REAL :: smallValue       !  a small value
  REAL :: wood             !  wood biomass (kgC m-2)
  REAL :: wt_ext_tile_ec(land_pts,nzsoilC,npft)  !  fraction of transpiration extracted
!                                                 from each ECOSSE layer
  LOGICAL :: empty         !  true when there is no more n to pay for. 
  LOGICAL :: unmetDemand   !  true while there is still demand for N

!-------------------------------------------------------------------------------
! Local array variables.
!-------------------------------------------------------------------------------

  INTEGER :: cheapRank(nzsoilC)   !  ranking of layers (cheapest=1)
  INTEGER :: iminCost(nzSoilC)    !  index (in cost) of lowest cost
  INTEGER :: mloc(1)              !  used to hold result of minloc

  REAL :: avail_nh4(land_pts,nzSoilC) !  amount of NH4 in the soil that is
!                                           available to plants (above crit)
!                                           (kgN m-2 layer-1)
  REAL :: avail_no3(land_pts,nzSoilC) !  amount of NO3 in the soil that is
!                                           available to plants (above crit)
!                                           (kgN m-2 layer-1)
!                                         (kgC m-2 per sub-step)
  REAL :: cost_of_n(nzsoilC)          !  cost of cheapest form of N in each soil
!                                        layer (kgC kgN-1). 
  REAL :: cost(nzSoilC,ncost)         !  cost of N via each process (kgC kgN-1)
  REAL :: dz(land_pts,nzSoilC)        !  ECOSSE soil layer thicknesses on a grid (m)
  REAL :: nh4_uptake(nzsoilC)         !  N resulting from uptake of NH4 (kgN m-2)
  REAL :: no3_uptake(nzsoilC)         !  N resulting from uptake of NO3 (kgN m-2)
  REAL :: no3_conc(land_pts,nzsoilC)  !  concentration of available NO3 in soil water
!                                        (kgN/kgH2O)
  REAL :: nh4_conc(land_pts,nzsoilC)  !  concentration of available NH4 in soil water
!                                        (kgN/kgH2O)
  REAL :: rootFrac_ec(npft,nzSoilC)   !  fraction of roots in each ECOSSE layer
  REAL :: smc(land_pts,nzSoilC)       !  soil moisture content in ECOSSE layers (kg m-2)
  REAL :: theta(land_pts,sm_levels)   !  volumetric soil moisture content
  REAL :: tsoilDegC(land_pts,nzSoilC) !  soil temperature on ECOSSE layers (degC)
  REAL :: zbotEC(nzSoilC)             !  depth to bottom of each ECOSSE layer (m)
  REAL :: zbotSMC(nzSoilC)            !  depth to bottom of each soil moisture
!                                        (JULES) layer (m)

  LOGICAL :: ranked(nzsoilC)        ! TRUE when a layer has been ranked

!-------------------------------------------------------------------------------
! Initialise output fluxes.
!-------------------------------------------------------------------------------
  gb_no3_uptake(:,:) = 0.0
  gb_nh4_uptake(:,:) = 0.0
  Nactive(:,:) = 0.0
  Ndemand(:,:) = 0.0
  Nfix(:,:) = 0.0
  Npassive(:,:) = 0.0
  Nuptake(:,:) = 0.0
  npp_Nactive(:,:) = 0.0
  npp_Nfix(:,:) = 0.0
  npp_Nretrans(:,:) = 0.0
  npp_Nuptake(:,:) = 0.0

!--------------------------------------------------------------------------
! Get a small number.
!--------------------------------------------------------------------------
!  smallValue = EPSILON( n_falling_leaves )  !  EPSILON not necessarily very small...
  smallValue = 1.0e-12

!--------------------------------------------------------------------------   
! Initialise leaf litter C:N to leaf value.
!-------------------------------------------------------------------------- 
  DO ft=1,npft
    DO j=1,tile_pts(ft)
      l = tile_index(j,ft)  
      leaf_litter_cn_ratio(l,ft) = 1.0 / nl0(ft)
    ENDDO
  ENDDO

!-------------------------------------------------------------------------------
! Map root and extraction variables from the soil moisture layers to the ECOSSE
! layers. More sophisticated schemes could be used.
!-------------------------------------------------------------------------------

! Calculate depth to bottom of each soil moisture (JULES) layer.
  zbotSmc(1) = dzsoil(1)
  DO iz=2,sm_levels
    zbotSmc(iz) = zbotSmc(iz-1) + dzsoil(iz)
  ENDDO

! Calculate depth to bottom of each ECOSSE layer.
  zbotEC(1) = dzSoilC(1)
  DO iz=2,nzSoilC
    zbotEC(iz) = zbotEC(iz-1) + dzSoilC(iz)
  ENDDO

! The following assumes that the ECOSSE layers are thinner than the soil layers.
! It also assumes that the total depths of the two soil columns are equal (so
! that both fields sum to 1 over the depths).
! More robust code would be better!
  j1=1
  DO iz=1,sm_levels

!   Find the ECOSSE layer in which the bottom of this soil layer lies.
    j2 = findLayer( zbotSMC(iz),zbotEC(:) )

!   For all ECOSSE layers covered by this soil moisture layer, set value
!   so that sum equals that in the soil moisture layer.
    DO jz=j1,j2
      rootFrac_ec(:,jz) = rootFrac(:,iz) / REAL(j2-j1+1)
      wt_ext_tile_ec(:,jz,:) = wt_ext_tile(:,iz,:) / REAL(j2-j1+1)
    ENDDO

!   Top of next section is bottom of this section.
    j1 = j2 + 1

  ENDDO

!-------------------------------------------------------------------------------
! Calculate volumetric soil moisture in each soil layer.
!-------------------------------------------------------------------------------
  DO iz=1,sm_levels
    DO l=1,land_pts
      theta(l,iz) = ( sthu(l,iz)+sthf(l,iz) ) * smvcst(l,iz)
    ENDDO
  ENDDO

!-------------------------------------------------------------------------------
! Calculate total soil moisture and temperature in each ECOSSE layer.
!-------------------------------------------------------------------------------
  dz(:,:) = SPREAD( dzSoilC(:),1,land_pts )
  smc(:,:) = ecosseLayers( land_pts,theta ) * dz(:,:) * rho_water
  tsoilDegC(:,:) = ecosseLayers( land_pts,t_soil ) - zeroDegC


!-------------------------------------------------------------------------------
! Initialise available soil N pools and concentration of N in soil water.
! N is assumed immobile and unavailable in dry soils.
! At present, dry means completely dry!
! Using the total soil water and soil N to calculate concentration in this way
! is equivalent to assuming concentration of N is same in liquid and ice and
! that only N in liquid is available.
!-------------------------------------------------------------------------------
  DO iz=1,nzSoilC
    DO l=1,land_pts
      IF ( smc(l,iz) > smallValue ) THEN
        avail_nh4(l,iz) = ammnSI(l,iz) - critSI(l,iz)
        avail_no3(l,iz) = soilNSI(l,iz) - critSI(l,iz)
        nh4_conc(l,iz) = avail_nh4(l,iz) / smc(l,iz)
        no3_conc(l,iz) =  avail_no3(l,iz) / smc(l,iz)
      ELSE
        avail_nh4(l,iz) = 0.0
        avail_no3(l,iz) = 0.0
        nh4_conc(l,iz) = 0.0
        no3_conc(l,iz) = 0.0
      ENDIF
    ENDDO
  ENDDO

!--------------------------------------------------------------------------
! Calculate multiplier to convert from amount per FUN timestep to per second.
!--------------------------------------------------------------------------
  multi = 1.0 / REAL( fun_period * isecInDay )

!-------------------------------------------------------------------------------
! Soil N is assumed to be uniform across the gridbox (we have a gridbox mean
! value), so all PFTs have access to the same resource. In theory this means
! that any PFT can access and extract all the N, even although the fractional
! cover of the PFT might be relatively small.
!
! Done this way, the earlier PFTs get preferential access to N (which, once
! extracted, will not be available to later PFTs). So as to avoid favouring
! these PFTs, the order in which the PFTs are considered changes between
! calls, using a counter of number of calls. (An alternative would be to use
! random number generator.)
! Note that if the sub-step loop was on the outside (and number of substeps>1),
! the order of the PFTs would be less important...but then all costs need to
! be in land_pts arrays, and generally takes more work to keep track of status
! within step loop.
!
! Calculations are carried out using fluxes for the fractional area in
! question, then converted to effect on gridbox mean by multiplying by frac.

!------------------------------------------------------------------------------- 
! Calculate which PFT to consider first.
!-------------------------------------------------------------------------------

  ft = MOD( funCount, npft )
  IF ( ft == 0 ) ft = npft

  DO ipft=1,npft

    IF ( ft > npft ) ft = 1

    DO J=1,TILE_PTS(ft)
      L=TILE_INDEX(J,ft)

      IF ( npp_acc_fun(l,ft) > 0.0 ) THEN

!--------------------------------------------------------------------------
!       There is a demand for N.
!--------------------------------------------------------------------------
     
!--------------------------------------------------------------------------
!       Initialise accumulations for diagnostics.
!--------------------------------------------------------------------------
        n_active_acc = 0.0
        n_fix_acc = 0.0
        n_passive_acc = 0.0
        n_retrans_acc = 0.0
        npp_active_acc = 0.0
        npp_fix_acc = 0.0
        npp_retrans_acc = 0.0
         
!-------------------------------------------------------------------------------
!       Calculate LAI and C of vegetation.
!-------------------------------------------------------------------------------
!       Calculate LAI from the vegetation height 
        lai_bal = ( a_ws(ft) * eta_sl(ft) * canht_ft(l,ft) / a_wl(ft) )  &
                      **  ( 1.0 / (b_wl(ft)-1.0) )
        lai = lai_bal 
        IF ( can_rad_mod == 1) THEN
          fpar = (1.0 - EXP(-kpar(ft)*lai)) / kpar(ft)
          nl = ( fpar/ lai) * nl0(ft)             
          nl_bal = (1.0 - EXP(-kpar(ft)*lai_bal)) / (kpar(ft)*lai_bal) * nl0(ft)
        ELSE
          nl = nl0(ft)
          nl_bal = nl0(ft)
        ENDIF                   
        leaf = sigl(ft) * lai_bal
        root = leaf
        wood = a_wl(ft) * ( lai_bal**b_wl(ft) )
          
!--------------------------------------------------------------------------
!       Calculate the current total nitrogen content of the vegetation.
!--------------------------------------------------------------------------     
        n_leaf = nl * sigl(ft) * lai
        n_root = nr_nl(ft) * nl_bal * root
        n_wood = ns_nl(ft) * nl_bal * eta_sl(ft) * canht_ft(l,ft) * lai
        currentCN  = (leaf+root+wood) / (n_leaf+n_root+n_wood)

!--------------------------------------------------------------------------
!       Save the N demand diagnostic.
!--------------------------------------------------------------------------
        Ndemand(l,ft) = npp_acc_fun(l,ft) /  &
                        ( currentCN * REAL(fun_period*isecInDay) )
    
!--------------------------------------------------------------------------
!       Calculate NPP per step of the integration, and the associated N demand.
!--------------------------------------------------------------------------
        npp_step = npp_acc_fun(l,ft) / REAL(nstep)
        n_demand_step = npp_step / currentCN

!--------------------------------------------------------------------------
!       Initialise the remaining, unmatched NPP.
!--------------------------------------------------------------------------
        npp_remaining = npp_acc_fun(l,ft)

!--------------------------------------------------------------------------  
!       Calculate C in litter over timestep, and the minimum allowed N in
!       leaf litter (to prevent litter C:N from becoming very large).
!--------------------------------------------------------------------------   
        c_turnover = leaf * g_leaf_phen(l,ft) * fun_period / 360.0
        n_litter_min = c_turnover / litter_cn_max

!--------------------------------------------------------------------------   
!       Calculate available pool of N (including minimum allowed store) in
!       the leaves that are "turning over" (dropping) in this timestep.
!--------------------------------------------------------------------------  
        n_falling_leaves = sigl(ft) * lai * g_leaf_phen(l,ft) *  &
                REAL(fun_period) / ( leaf_litter_cn_ratio(l,ft) * 360.0 )

!--------------------------------------------------------------------------
!       Calculate the cost of fixing. This does not vary within a timestep.
!--------------------------------------------------------------------------
        DO iz=1,nzSoilC
          cost(iz,icostFix) = fun_cost_fix( fixer(ft),a_fix,b_fix,c_fix    &
                      ,big_cost,rootFrac_ec(ft,iz),s_fix,tsoilDegC(l,iz) )
        ENDDO

!--------------------------------------------------------------------------
!       Start the sub-timestep integration loop. This loop was introduced
!       as a conceptually simple way in which to account for the fact that
!       the cost of N from a pool increases continuously as N is extracted
!       from the pool, which can result in another pool becoming cheaper
!       part way through the timestep. Each step in the loop attempts to
!       satisfy part of the demand by extracting from the cheapest store.
!       At the end of the step, the stores and costs are updated, for use
!       in the following step.
!
!       The number of sub-steps could be made a function of available store
!       since a small extraction from a large store occurs at relatively
!       constant cost, and it is probably unnecessary to use the sub-steps
!       to update the cost. e.g. if demand<10% of store, use a single step.
!       Also...if passive uptake is sufficient to meet demand, we could again
!       use a single step...but probably rarely occurs.
!--------------------------------------------------------------------------

        DO istep=1,nstep

!-------------------------------------------------------------------------------
!         Initialise for this PFT, at this point, this sub-step.
!-------------------------------------------------------------------------------
          dn = 0.0
          dnpp = 0.0
          n_passive_step  = 0.0
          empty = .FALSE.
          nh4_uptake(:) = 0.
          no3_uptake(:) = 0.
          unmetDemand = .TRUE.

!--------------------------------------------------------------------------
!         Passive N uptake by transpiration stream
!--------------------------------------------------------------------------
          DO iz=1,nzSoilC

!           Calculate passive uptake from this layer.
            nh4_uptake(iz) = nh4_conc(l,iz) * transp_acc_fun(l,ft) *  &
                                       wt_ext_tile_ec(l,iz,ft) / REAL(nstep)
            no3_uptake(iz) = no3_conc(l,iz) * transp_acc_fun(l,ft) *  &
                                       wt_ext_tile_ec(l,iz,ft) / REAL(nstep)
!           Limit passive uptake to the available N in this layer.
!           This is the available N across the gridbox (not just this frac).
            nh4_uptake(iz) = MIN( nh4_uptake(iz), avail_nh4(l,iz) )
            no3_uptake(iz) = MIN( no3_uptake(iz), avail_no3(l,iz) )

!           Add to total uptake from column by this PFT.
            n_passive_step = n_passive_step + nh4_uptake(iz) + no3_uptake(iz)

          ENDDO  !  iz

          IF ( n_demand_step <= n_passive_step ) THEN

!--------------------------------------------------------------------------
!           Passive uptake exceeds requirement. Return excess N to soil
!           (via whatever mechanisms, e.g. diffusion). 
!--------------------------------------------------------------------------
            unmetDemand = .FALSE.
            excess = n_passive_step - n_demand_step
            
!           Return to the soil by reducing the uptake rates.
!           Add to NO3 and NH4 pools, according to calculated extraction
!           from each in each layer.
            DO iz=1,nzSoilC
              nh4_uptake(iz) = nh4_uptake(iz) -  &
                   excess * nh4_uptake(iz) / n_passive_step
              no3_uptake(iz) = no3_uptake(iz)  -  &
                   excess * no3_uptake(iz) / n_passive_step
            ENDDO
            n_passive_step = n_demand_step

          ENDIF  !  n_demand_step <= n_passive_step

!--------------------------------------------------------------------------
!         Update N taken up and available N in soil.
!--------------------------------------------------------------------------
          n_passive_acc = n_passive_acc + n_passive_step
          DO iz=1,nzSoilC
            avail_nh4(l,iz) = avail_nh4(l,iz) - nh4_uptake(iz) * frac(l,ft)
            avail_no3(l,iz) = avail_no3(l,iz) - no3_uptake(iz) * frac(l,ft)
          ENDDO

!--------------------------------------------------------------------------
!         Update concentration of N in soil water.
!--------------------------------------------------------------------------
          DO iz=1,nzSoilC
            IF ( smc(l,iz) > smallValue ) THEN
              nh4_conc(l,iz) = avail_nh4(l,iz) / smc(l,iz)
              no3_conc(l,iz) =  avail_no3(l,iz) / smc(l,iz)
            ELSE
!             N immobile in dry soil.
              nh4_conc(l,iz) = 0.0
              no3_conc(l,iz) = 0.0
            ENDIF
          ENDDO

!--------------------------------------------------------------------------
!         If passive uptake is insufficient, consider fixation, active
!         uptake and retranslocation.
!--------------------------------------------------------------------------
          IF ( unmetDemand ) THEN

!--------------------------------------------------------------------------
!           Calculate NPP still to pay for after passive uptake.
!--------------------------------------------------------------------------
            npp_remaining =  npp_remaining - n_passive_step * currentCN
            npp_remaining_step =  npp_step - n_passive_step * currentCN

!--------------------------------------------------------------------------
!           Calculate the costs of the various possible sources.
!           The cost of nitrogen fixation does not vary within a timestep,
!           and has been calculated outside this loop.
!--------------------------------------------------------------------------
            
!--------------------------------------------------------------------------
!           Costs of active uptake.
!--------------------------------------------------------------------------
            DO iz=1,nzSoilC
              cost(iz,icostActiveNH4) = fun_cost_active( avail_nh4(l,iz)  &
                           ,big_cost,dzSoilC(iz),kc_active,kn_active      &
                           ,root,rootFrac_ec(ft,iz),smallValue )
              cost(iz,icostActiveNO3) = fun_cost_active( avail_no3(l,iz)  &
                           ,big_cost,dzSoilC(iz),kc_active,kn_active      &
                           ,root,rootFrac_ec(ft,iz),smallValue )
            ENDDO
            
!--------------------------------------------------------------------------
!           Cost of retranslocation.
!           This is set to the same value for all layers for ease of use,
!           although in fact it does not depend on layer.
!--------------------------------------------------------------------------
!           This is also the demand for future N, to replace this leaf...
!           'nitrogen maintenance demand'.

            cost(:,icostRetrans) = fun_cost_retrans( nzSoilC,big_cost    &
                                   ,kr_resorb,n_falling_leaves,smallValue )
           
!--------------------------------------------------------------------------
!           Decide which source of N is cheapest in each layer.
!--------------------------------------------------------------------------
            DO iz=1,nzSoilC
              mloc(1:1) = MINLOC( cost(iz,:) )
              iminCost(iz) = mloc(1)
              cost_of_n(iz) = cost(iz,iminCost(iz))
            ENDDO
            
!--------------------------------------------------------------------------
!           Rank layers in terms of cost of nitrogen uptake, cheapest first.
!--------------------------------------------------------------------------
            ranked(:) = .FALSE.
            DO i=1,nzSoilC
              mloc(1:1) = MINLOC( cost_of_n(:), .NOT. ranked(:) )
              cheapRank(i) = mloc(1)
              ranked(mloc(1)) = .TRUE.
            ENDDO

!--------------------------------------------------------------------------
!           Spend C on extracting N.
!--------------------------------------------------------------------------

!           Use the layer with the cheapest N.
            iz = cheapRank(1)

!           If the cheapest N is very expensive, stores are empty.
            IF ( cost_of_n(iz) > big_cost - 0.1 ) THEN

              empty = .TRUE.

            ELSE

!             Identify the process involved with this cheapest N.
              icost = iminCost(iz)

!             Calculate NPP decrement and N increment.
              dnpp = npp_remaining_step / ( currentCN / cost(iz,icost) + 1.)
              dn = dnpp / cost(iz,icost)
!--------------------------------------------------------------------------
!             Load values into process-specific variables and limit
!             increments if store exhausted. Note that the fluxes and
!             increments calculated so far are for this fraction, and need
!             to be scaled to get effect on gridbox scale.
!--------------------------------------------------------------------------

              IF ( icost == icostActiveNH4 ) THEN
!--------------------------------------------------------------------------
!               Active NH4 uptake.
!               Decrement available store and update concentration.
!               If store insufficient, reduce uptake and NPP increment.
!--------------------------------------------------------------------------
                IF ( dn*frac(l,ft) > avail_nh4(l,iz) ) THEN
                  dn = avail_nh4(l,iz) / frac(l,ft)
                  dnpp = dn * cost(iz,icost)
                ENDIF
                avail_nh4(l,iz) = avail_nh4(l,iz) - dn*frac(l,ft)
                IF ( smc(l,iz) > smallValue ) THEN
                  nh4_conc(l,iz) = avail_nh4(l,iz) / smc(l,iz)
                ELSE
                  nh4_conc(l,iz) = 0.0
                ENDIF
                n_active_acc = n_active_acc + dn
                npp_active_acc = npp_active_acc + dnpp
                nh4_uptake(iz) = nh4_uptake(iz) + dn

              ELSEIF ( icost == icostActiveNO3 ) THEN

!--------------------------------------------------------------------------
!               Active NO3 uptake.
!               Decrement available store and update concentration.
!               If store insufficient, reduce uptake and NPP increment.
!--------------------------------------------------------------------------
                IF ( dn*frac(l,ft) > avail_no3(l,iz) ) THEN
                  dn = avail_no3(l,iz) / frac(l,ft)
                  dnpp = dn * cost(iz,icost)
                ENDIF
                avail_no3(l,iz) = avail_no3(l,iz) - dn*frac(l,ft)
                IF ( smc(l,iz) > smallValue ) THEN
                  no3_conc(l,iz) = avail_no3(l,iz) / smc(l,iz)
                ELSE
                  no3_conc(l,iz) = 0.0
                ENDIF
                n_active_acc = n_active_acc + dn
                npp_active_acc = npp_active_acc + dnpp
                no3_uptake(iz) = no3_uptake(iz) + dn

              ELSEIF ( icost == icostFix ) THEN
!--------------------------------------------------------------------------
!               Nitrogen fixer uptake.
!--------------------------------------------------------------------------
                n_fix_acc = n_fix_acc + dn
                npp_fix_acc = npp_fix_acc + dnpp

!               Note that fixing will remain the cheapest store for all
!               subsequent steps (effectively it is accessing an infinite
!               store of gaseous N2), so explicit integration over each of
!               those individual steps could be avoided and NPP summed over
!               all remaining steps. But this has not been coded.

              ELSE
!-------------------------------------------------------------------------- 
!               Retranslocation.
!               If insufficient N in lost leaves, reduce retranslocation
!               and NPP increment.
!-------------------------------------------------------------------------- 
                IF ( dn > n_falling_leaves - n_litter_min ) THEN
                  dn = n_falling_leaves - n_litter_min
                  dnpp = dn * cost(iz,icost)
                ENDIF
                n_retrans_acc = n_retrans_acc + dn
                npp_retrans_acc = npp_retrans_acc + dnpp
!               Update N in falling leaves.
                n_falling_leaves = n_falling_leaves - dn

              ENDIF  !  processes

!--------------------------------------------------------------------------
!             Decrement unmatched NPP over FUN timestep.
!-------------------------------------------------------------------------- 
              npp_remaining = npp_remaining - dnpp

!-------------------------------------------------------------------------- 
!             Update costs, ready for next step.
!-------------------------------------------------------------------------- 
              IF ( istep < nstep ) THEN

                IF ( icost == icostActiveNH4 ) THEN
  
                  cost(iz,icostActiveNH4) =                            &
                        fun_cost_active( avail_nh4(l,iz)               &
                           ,big_cost,dzSoilC(iz),kc_active,kn_active   &
                           ,root,rootFrac_ec(ft,iz),smallValue )

                ELSEIF ( icost == icostActiveNO3 ) THEN

                  cost(iz,icostActiveNO3) =                            &
                        fun_cost_active( avail_no3(l,iz)               &
                           ,big_cost,dzSoilC(iz),kc_active,kn_active   &
                           ,root,rootFrac_ec(ft,iz),smallValue )
  
                ELSEIF ( icost == icostRetrans ) THEN

                  cost(:,icostRetrans) = fun_cost_retrans(  &
                             nzSoilC,big_cost,kr_resorb,n_falling_leaves  &
                            ,smallValue )
  
                ENDIF
  
!-------------------------------------------------------------------------- 
!               Decide which source of N is now cheapest in this layer.
!               Decide whether to update ranking - there's no need if the
!               cheapest layer is still the cheapest!
!-------------------------------------------------------------------------- 
                mloc(1:1) = MINLOC( cost(iz,:) )
                iminCost(iz) = mloc(1)
                cost_of_n(iz) = cost(iz,iminCost(iz))
  
                IF ( cost_of_n(iz) > cost_of_n(cheapRank(2)) ) THEN
!                 We only have to insert a single value...but sort whole list!
                  ranked(:) = .FALSE.
                  DO i=1,nzSoilC
                    mloc(1:1) = MINLOC( cost_of_n(:), .NOT. ranked(:) )
                    cheapRank(i) = mloc(1)
                    ranked(mloc(1)) = .TRUE.
                  ENDDO
                ENDIF

              ENDIF  !  istep<nstep

            ENDIF !  cost>big_cost (empty)

!--------------------------------------------------------------------------
!           Update C fluxes.
!--------------------------------------------------------------------------
            IF ( empty ) THEN
!             All N in the pools has been extracted, so we can get no N
!             back whatever we pay.
              npp_acc_fun(l,ft) =  npp_acc_fun(L,ft) - npp_remaining
              resp_p_ft(l,ft) = resp_p_ft(l,ft) + npp_remaining
!             Leave integration loop (steps).
              EXIT
            ELSE
!             Decrement accumulated NPP by amount spent on N.
!             Add C spent on N uptake to plant respiration. 
              npp_acc_fun(l,ft) =  npp_acc_fun(l,ft) - dnpp
              resp_p_ft(l,ft) = resp_p_ft(l,ft) + dnpp
            ENDIF

          ENDIF  !  unmetDemand

!--------------------------------------------------------------------------
!         Increment gridbox mean extraction rates.
!--------------------------------------------------------------------------
          DO iz=1,nzSoilC
            gb_nh4_uptake(l,iz) = gb_nh4_uptake(l,iz) +  &
                                         nh4_uptake(iz) * frac(l,ft)
            gb_no3_uptake(l,iz) = gb_no3_uptake(l,iz) +  &
                                       no3_uptake(iz) * frac(l,ft)
          ENDDO

        ENDDO  !  steps

!--------------------------------------------------------------------------
!       Update the C:N ratio of the litter.
!--------------------------------------------------------------------------
        IF ( c_turnover > 0.0 ) leaf_litter_cn_ratio(l,ft) =  &
                 c_turnover / n_falling_leaves

!--------------------------------------------------------------------------
!       Increment the gridbox mean plant respiration.
!--------------------------------------------------------------------------
        resp_p(l) = resp_p(l) + frac(l,ft) * resp_p_ft(l,ft)

!--------------------------------------------------------------------------
!       Save diagnostics.
!--------------------------------------------------------------------------
        Nactive(l,ft) = n_active_acc * multi
        Nfix(l,ft) = n_fix_acc * multi
        Npassive(l,ft) = n_passive_acc * multi
        Nretrans(l,ft) = n_retrans_acc * multi
        Nuptake(l,ft) = Nactive(l,ft) + Nfix(l,ft) + Npassive(l,ft) +  &
                            Nretrans(l,ft)
        npp_Nactive(l,ft) = npp_active_acc * multi
        npp_Nfix(l,ft) = npp_fix_acc * multi
        npp_Nretrans(l,ft) = npp_retrans_acc * multi
        npp_Nuptake(l,ft) = npp_Nactive(l,ft) + npp_Nfix(l,ft) +  &
                              npp_Nretrans(l,ft)

      ENDIF !   npp_acc_fun > 0.0

    ENDDO  !  points

!--------------------------------------------------------------------------
!   Increment PFT index.
!--------------------------------------------------------------------------
    ft = ft + 1

  ENDDO  !  pft

  END SUBROUTINE fun_uptake_n

!###############################################################################
!###############################################################################
!###############################################################################

  FUNCTION fun_cost_active( availN,big_cost,dzSoilC,kc_active,kn_active  &
                           ,root,rootFrac,smallValue )  &
           RESULT ( cost_of_n )

!-----------------------------------------------------------------------
! Function to calculate the cost of active uptake of NH4 or NO3 from a layer.
!-----------------------------------------------------------------------

  IMPLICIT NONE

! Function result.
  REAL :: cost_of_n   !  cost of active uptake of NH4 or NO3 (kgC/kgN)

!--------------------------------------------------------------------------
! Scalar arguments with intent(in).
!--------------------------------------------------------------------------
  REAL, INTENT(in) :: availN   !  amount of N (as NH4 or NO3 ) in the soil
!                                 that is available to plants (kgN m-2)
  REAL, INTENT(in) :: big_cost !  an arbitrary large cost (kgC/kgN)
  REAL, INTENT(in) :: dzSoilC  !  thickness of soil layer (m)
  REAL, INTENT(in) :: kc_active!  Constant for cost of active uptake (kgC m-2)
  REAL, INTENT(in) :: kn_active!  Constant for cost of active uptake (kgC m-2)
  REAL, INTENT(in) :: root     !  root biomass (kgC m-2)
  REAL, INTENT(in) :: rootFrac !  fraction of roots that are in this layer
  REAL, INTENT(in) :: smallValue !  a small number

!--------------------------------------------------------------------------
  IF ( rootFrac > 1.0e-4 ) THEN

    IF( availN > smallValue ) THEN
      cost_of_n =  kn_active * dzsoilC / availN * kc_active /  &
            ( rootFrac * root / dzSoilC )
    ELSE
      cost_of_n = big_cost   !  arbitrary large number
    ENDIF    

  ELSE

!   There are very few roots in this layer. Set a high cost.
    cost_of_n = big_cost

  ENDIF

  END FUNCTION fun_cost_active

!###############################################################################
!###############################################################################

  FUNCTION fun_cost_fix( fixer,a_fix,b_fix,c_fix,big_cost,rootFrac,s_fix  &
                        ,tsoilDegC )   RESULT ( cost_of_n )

!-----------------------------------------------------------------------
! Function to calculate the cost of fixing N by nodules.
!-----------------------------------------------------------------------

  IMPLICIT NONE

! Function result.
  REAL :: cost_of_n   !  cost of fixing N (kgC/kgN)

!--------------------------------------------------------------------------
! Scalar arguments with intent(in).
!--------------------------------------------------------------------------
  INTEGER, INTENT(in) :: fixer  !  flag indicating if plant is a fixer
!                                  1=yes, otherwise no.
  REAL, INTENT(in) :: a_fix     !  As in Houlton et al. (Nature) 2008
  REAL, INTENT(in) :: b_fix     !  As in Houlton et al. (Nature) 2008 
  REAL, INTENT(in) :: c_fix     !  As in Houlton et al. (Nature) 2008 
  REAL, INTENT(in) :: big_cost  !  an arbitrary large cost (kgC/kgN)
  REAL, INTENT(in) :: rootFrac  !  fraction of roots that are in this layer
  REAL, INTENT(in) :: s_fix     !  Inverts Houlton et al. 2008 and constrains
!                                  between 7.5 and 12.5 
  REAL, INTENT(in) :: tsoilDegC !  soil temperature (degrees Celsius)

!--------------------------------------------------------------------------
  IF ( fixer == 1 ) THEN

    IF ( rootFrac > 1.0e-4 ) THEN

      cost_of_n = s_fix * EXP( a_fix + b_fix * tsoilDegC        &
                       * ( 1.0 - 0.5 * tsoilDegC / c_fix ) ) - 2.0 * s_fix

    ELSE

!     There are very few roots in this layer.
!     Set cost to an arbitrary high value.
      cost_of_n = big_cost

    ENDIF

  ELSE

!   Plant is not a fixer. Set cost to an arbitrary high value.
    cost_of_n = big_cost

  ENDIF

  END FUNCTION fun_cost_fix

!###############################################################################
!###############################################################################
  FUNCTION fun_cost_retrans( nzSoilC,big_cost,kr_resorb,n_falling_leaves  &
                            ,smallValue )  &
           RESULT ( cost_of_n )

!-----------------------------------------------------------------------
! Function to calculate the cost of retranslocation of N from leaves.
!-----------------------------------------------------------------------

  IMPLICIT NONE

! Function result.
  REAL :: cost_of_n(nzSoilC)   !  cost of retranslocation of N (kgC/kgN)
!    This is an array with size equal to the number of soil layers,
!    although the cost is not associated with layers! An array rather than
!    scalar is used to allow this cost to be compared in another part of
!    FUN with other costs that do vary between layers.

!--------------------------------------------------------------------------
! Scalar arguments with intent(in).
!--------------------------------------------------------------------------
  INTEGER, INTENT(in) :: nzSoilC       !  number of layers
  REAL, INTENT(in) :: big_cost         !  an arbitrary large cost (kgC/kgN)
  REAL, INTENT(in) :: kr_resorb        !  Cost of retranslocation of N 
!                                           from falling leaves (kgC/kgN)
  REAL, INTENT(in) :: n_falling_leaves !  N stored in leaves that are
!                                           about to fall off (kgN m-2)
  REAL, INTENT(in) :: smallValue       !  a small number

!--------------------------------------------------------------------------

  IF ( n_falling_leaves > smallValue ) THEN

    cost_of_n(:) = kr_resorb / n_falling_leaves

  ELSE

!   Little N available. Set cost to an arbitrary high value.
    cost_of_n(:) = big_cost

  ENDIF

  END FUNCTION fun_cost_retrans

!###############################################################################
!###############################################################################

  END MODULE fun_mod

!###############################################################################
!###############################################################################
!###############################################################################
